#!/bin/sh -e
### BEGIN INIT INFO
# Provides:          rfctl
# Required-Start:    
# Required-Stop:     
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: 
# Description:       Record the last status of radios when shutdown, and bring
#                    them back (turn on/off) again on resume.
# Author:            Kim Wong <kim.max@gmail.com>
### END INIT INFO

PATH="/bin:/sbin:/usr/bin:/usr/sbin"
RF_IDENTIFIERS="wifi wlan bluetooth uwb ultrawideband wimax wwan gps fm"
STATUS_FILE="/var/local/rfctl"
LOG="true"
LOGFILE="/var/log/rfctl.log"

do_command ()
{
	if [ "$LOG" = "true" ] ; then
		echo $@ >> $LOGFILE
	fi
	$@
}

case $1 in
	start)
		if [ "$LOG" = "true" ] ; then
			echo ">>>>>> START <<<<<<" >> $LOGFILE
			date +"%F %T.%N" >> $LOGFILE
		fi

		if [ -e /var/local/rfctl ] ; then
			for i in $RF_IDENTIFIERS ; do
				SOFT_BLOCKED=`grep "$i" /var/local/rfctl | sed -e "s/$i:\(.*\)/\1/"`
				if [ "$SOFT_BLOCKED" = "yes" ] ; then
					do_command rfkill block $i
				elif [ "$SOFT_BLOCKED" = "no" ] ; then
					do_command rfkill unblock $i
				fi
			done
		fi

		echo "rfctl started."
		;;

	stop)
		if [ "$LOG" = "true" ] ; then
			echo ">>>>>> STOP <<<<<<" >> $LOGFILE
			date +"%F %T.%N" >> $LOGFILE
			rfkill list all | sed -e "s/^/\t/" >> $LOGFILE
		fi

		echo -n > /var/local/rfctl

		for i in $RF_IDENTIFIERS ; do
			S2=`rfkill list $i | sed -e 'N;N;s/\n/;;/g;s/\t//g;s/^[0-9]*: \S*: .*;;Soft blocked: \(.*\);;Hard blocked: \(.*\)$/\1:\2/;q'`
			SOFT_BLOCKED=`echo $S2 | sed -e 's/\(.*\):\(.*\)/\1/'`
			HARD_BLOCKED=`echo $S2 | sed -e 's/\(.*\):\(.*\)/\2/'`

			if [ -z $SOFT_BLOCKED ] ; then
				SOFT_BLOCKED='unknown'
			fi
			if [ -z $HARD_BLOCKED ] ; then
				HARD_BLOCKED="unknown"
			fi

			# Record status on hard-blocked is "no", only.
			#
			# The system will not record the status of soft-block
			# when user hard-block the radios. And when the user
			# turnning on the radios after he/she hard-blocked the
			# radios, the system will always set soft-blocked to
			# "no".
			if [ $HARD_BLOCKED = "no" ] ; then
				echo "$i:$SOFT_BLOCKED" >> /var/local/rfctl
			fi
		done

		echo "rfctl stopped."
		;;

	install)
		if [ "$LOG" = "true" ] ; then
			echo ">>>>>> INSTALL <<<<<<" >> $LOGFILE
			date +"%F %T.%N" >> $LOGFILE
		fi

		cp rfctl.sh /etc/init.d/rfctl
		chmod +x /etc/init.d/rfctl

		for RUNLEVEL in 0 1 2 3 4 5 6 S ; do
			cd /etc/rc$RUNLEVEL.d
			if [ -e S39rfctl ] ; then
				rm S39rfctl
			fi
			ln -s ../init.d/rfctl S39rfctl
		done

		echo "rfctl is installed."
		;;

	uninstall)
		if [ "$LOG" = "true" ] ; then
			echo ">>>>>> UNINSTALL <<<<<<" >> $LOGFILE
			date +"%F %T.%N" >> $LOGFILE
		fi

		for RUNLEVEL in 0 1 2 3 4 5 6 S ; do
			if [ -e /etc/rc$RUNLEVEL.d/S39rfctl ] ; then
				rm /etc/rc$RUNLEVEL.d/S39rfctl
			fi
		done
		if [ -e /etc/init.d/rfctl ] ; then
			rm /etc/init.d/rfctl
		fi
		if [ -e /var/local/rfctl ] ; then
			rm /var/local/rfctl
		fi

		echo "rfctl is uninstalled."
		;;

	*)
		echo "Usage: /etc/init.d/rfctl {start|stop|install|uninstall}"
		exit 1;
		;;

esac

exit 0

# vim: set nobk ts=2 sw=2:
